/*
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 */

package org.ow2.orchestra.login.client.ui.impl;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.resources.client.ImageResource;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.uibinder.client.UiTemplate;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.DockLayoutPanel;
import com.google.gwt.user.client.ui.Frame;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.InlineLabel;
import com.google.gwt.user.client.ui.LayoutPanel;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.Widget;
import org.ow2.orchestra.login.client.ui.LoginView;

/**
 * @author Loic Albertin
 */
public class LoginViewImplShell extends Composite implements LoginView {


  public interface Presenter {
    void getUserName();

    void getConnectionTime();

    void logout();

    Frame getMainFrame();

  }

  LoginViewImplShell.Presenter presenter;

  public interface Resources extends LoginView.Resources {
    @Source("fullingDownScreen.png")
    ImageResource fullingDownScreen();

    @Source("fullingUpScreen.png")
    ImageResource fullingUpScreen();

    @Source("logout_icon.gif")
    ImageResource logoutButton();

    @Source({LoginView.Style.DEFAULT_CSS, LoginViewImplShell.Style.DEFAULT_CSS})
    LoginViewImplShell.Style style();

  }

  public interface Style extends LoginView.Style {
    static final String DEFAULT_CSS = "org/ow2/orchestra/login/client/ui/impl/shell.css";

    String infoLabelStyle();

    String infoFullScreenImage();

    String infoOrchestraLabel();

    String controlsPanel();

  }

  @UiField(provided = true)
  final LoginViewImplShell.Resources resources = GWT.create(LoginViewImplShell.Resources.class);

  @UiField
  Image logoutButton;
  @UiField
  Image fullScreenButton;
  @UiField
  InlineLabel connectionLabel;
  @UiField
  InlineLabel orchestraLabel;
  @UiField
  HorizontalPanel infoPanel;
  @UiField
  LayoutPanel contentPanel;
  @UiField
  DockLayoutPanel dockLayoutPanel;
  @UiField
  SimplePanel bannerPanel;

  private boolean isFullScreen = false;

  @UiTemplate("LoginViewImplShell.ui.xml")
  interface LoginViewImplShellUiBinder extends UiBinder<Widget, LoginViewImplShell> {
  }

  private static LoginViewImplShellUiBinder uiBinder = GWT.create(LoginViewImplShellUiBinder.class);

  public LoginViewImplShell() {
    resources.style().ensureInjected();
    initWidget(LoginViewImplShell.uiBinder.createAndBindUi(this));
  }

  public void setPresenter(LoginViewImplShell.Presenter presenter) {
    this.presenter = presenter;
  }

  public void setUserName(final String userName) {
    connectionLabel.setText(" connected as " + userName);
  }

  public void setMainFrame() {
    if (presenter != null) {
      contentPanel.add(presenter.getMainFrame());
    }

  }

  public void setConnectionTime(final String creationDate) {
    connectionLabel.setTitle(" connected since " + creationDate);
  }

  @UiHandler("logoutButton")
  void handleLogoutClick(ClickEvent e) {
    if (presenter != null) {
      presenter.logout();
    }
  }

  @UiHandler("fullScreenButton")
  public void handleFullScreenClick(ClickEvent event) {
    if (isFullScreen) {
      dockLayoutPanel.insertNorth(bannerPanel, resources.style().bannerPanelSize(), infoPanel);
      dockLayoutPanel.forceLayout();
      fullScreenButton.setResource(resources.fullingUpScreen());
      fullScreenButton.setTitle("Full Screen");
      orchestraLabel.setVisible(false);
      isFullScreen = false;
    } else {
      dockLayoutPanel.remove(bannerPanel);
      dockLayoutPanel.forceLayout();
      fullScreenButton.setResource(resources.fullingDownScreen());
      fullScreenButton.setTitle("Exit Full Screen");
      orchestraLabel.setVisible(true);
      isFullScreen = true;
    }
  }

  public void init() {

    if (presenter != null) {
      presenter.getUserName();
      presenter.getConnectionTime();
    }
    setMainFrame();
  }
}
